/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api;

import java.util.ArrayList;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * A single Author extension state listeners which delegates to other registered listeners.
 * This is useful in case you need to receive activated() events in more implementations.
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public class AuthorExtensionStateListenerDelegator implements AuthorExtensionStateListener {

  /**
   * The registered listeners
   */
  private java.util.List<AuthorExtensionStateListener> listeners = new ArrayList<AuthorExtensionStateListener>();

  /**
   * @see ro.sync.ecss.extensions.api.AuthorExtensionStateListener#activated(ro.sync.ecss.extensions.api.AuthorAccess)
   */
  public void activated(AuthorAccess authorAccess) {
    fireActivated(authorAccess);
  }

  /**
   * @see ro.sync.ecss.extensions.api.AuthorExtensionStateListener#deactivated(ro.sync.ecss.extensions.api.AuthorAccess)
   */
  public void deactivated(AuthorAccess authorAccess) {
    fireDeactivated(authorAccess);
  }
  
  /**
   * Add a new listener.
   * 
   * @param listener The new extension state listener.
   */
  public void addListener(AuthorExtensionStateListener listener){
    if(! listeners.contains(listener)) {
      listeners.add(listener);
    }
  }
  
  /**
   * Remove a listener.
   * 
   * @param listener The extension state listener to remove.
   */
  public void removeListener(AuthorExtensionStateListener listener){
    listeners.remove(listener);
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Author Extension State Listener Delegator";
  }
  
  /**
   * Fire extension activated.
   * @param authorAccess The author access
   */
  private void fireActivated(AuthorAccess authorAccess) {
    for (int i = 0; i < listeners.size(); i++) {
      listeners.get(i).activated(authorAccess);
    }
  }
  
  /**
   * Fire extension de activated
   * @param authorAccess The author access
   */
  private void fireDeactivated(AuthorAccess authorAccess) {
    for (int i = 0; i < listeners.size(); i++) {
      listeners.get(i).deactivated(authorAccess);
    }
  }
}
