/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * Gives access to translate keys a certain each framework to a target language.
 * 
 * The translation files should all be located in a separate directory (usually named "i18n") in the specific framework's directory.
 * This particular "i18n" directory should be explicitly added to the document type's class path list in the Document Type Edit dialog. 
 * 
 * Their names should be formatted like this:
 * 
 * <b>translation*.xml</b>
 * 
 * Only one translation file will be used to translate the keys to values.
 * 
 * How is this translation file detected?
 * 
 * The first file which matches the same GUI language that Oxygen uses in the Preferences will be used.
 * If no such file exists, the first encountered translation will be used.
 * If no such file exists, the methods will return the same key which is given for translation.
 * 
 * @since 14
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface AuthorResourceBundle {
  /**
   * Gets a message for the given key from this resource bundle.
   *
   * @param key The key for the desired string
   * @exception NullPointerException if <code>key</code> is <code>null</code>
   * @return the string for the given key
   */
  public String getMessage(String key);
  
  /**
   * Compose a message for the given description which may contain more than one keys referenced using editor variables.
   * The key contains "${i18n(key)}" editor variables which will be expanded.
   *
   * @param descriptionWithEditorVariables the key for the desired string.
   * 
   * Example:
   * 
   * The description: '${i18n(key1)} and ${i18n(key2)}' will be composed to:
   * 'value1 and value2' where 'value1' is obtained by calling getMessage('key1').
   * 
   * @exception NullPointerException if <code>descriptionWithEditorVariables</code> is <code>null</code>
   * @return the string for the given key
   */
  public String composeMessage(String descriptionWithEditorVariables);
}
