/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.callouts;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight;

/**
 * Provider for data that will be rendered as callouts, in Author mode, 
 * for custom persistent highlights. 
 * <br/>
 * The callouts are representations of Track Changes insert and delete highlights, 
 * review comment highlights and custom review highlights in Author mode.
 * <br/>
 * By default, the callouts visibility in Author mode is controlled from Oxygen
 * Preferences but it can be changed by using the {@link AuthorCalloutsController} methods.
 * <br/>
 * The callouts rendering provider can be set from 
 * {@link AuthorCalloutsController#setCalloutsRenderingInformationProvider(CalloutsRenderingInformationProvider)} 
 * 
 * @since 14
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class CalloutsRenderingInformationProvider {
  
  /**
   * Get the callout rendering information associated with a custom persistent 
   * highlight. The callout rendering information is requested only for that custom
   * persistent highlights for which the {@link #shouldRenderAsCallout(AuthorPersistentHighlight)}
   * returns <code>true</code>.
   * <br/>
   * The callouts are representations of Track Changes insert and delete highlights, 
   * review comment highlights and custom review highlights in Author mode.
   * 
   * @param highlight The Author persistent highlight. 
   * @return The callout rendering information associated with a custom persistent 
   * highlight or <code>null</code> if the highlight must not be rendered in Author as a
   * callout. 
   */
  public abstract AuthorCalloutRenderingInformation getCalloutRenderingInformation(AuthorPersistentHighlight highlight);
  
  /**
   * Asks if a custom persistent highlight should be rendered as a callout in the Author mode.
   * <br/>
   * The callouts are representations of Track Changes insert and delete highlights, 
   * review comment highlights and custom persistent highlights in Author mode.
   * <br/>
   * If this method returns <code>true</code>, the callout rendering information
   * for this callout must be provided by {@link #getCalloutRenderingInformation(AuthorPersistentHighlight)}
   * method.
   * <br/>
   * The implementation of this method must be fast, being called frequently.
   * 
   * @param highlight The Author custom persistent highlight. 
   * @return <code>true</code> if the highlight can be rendered as a callout in Author mode.
   */
  public abstract boolean shouldRenderAsCallout(AuthorPersistentHighlight highlight);
}
