/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.editor;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.CIValue;

/**
 * Arguments of the oxy_editor function as well as built-in values for some of 
 * these arguments.
 * 
 * @since 14.1
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface InplaceEditorCSSConstants {
  //////////////////////////////////////
  // CSS 'editor' extension function arguments
  /**
   * Indicates the editor that should be used to edit. One of TYPE_ constants or 
   * a class name. This is a shorthand to specify a built-in type of renderer 
   * and editor as opposed to using properties PROPERTY_RENDERER_CLASS_NAME, 
   * PROPERTY_SWING_EDITOR_CLASS_NAME and PROPERTY_SWT_EDITOR_CLASS_NAME.
   */
  String PROPERTY_TYPE = "type";
  /**
   * Indicates if we should edit an attribute value or the text value of the element.
   * The following values are accepted:
   * <ul>
   * <li>
   * To edit an attribute value the value of the property is "@attr_name".
   * </li>
   * <li>
   * To edit the text content the value should be equal with {@link #EDIT_TEXT_CONTENT}
   * </li>
   * <li>
   * To let the editor do the editing itself, the value should be equal with {@link #EDIT_CUSTOM}
   * </li>
   * </ul>
   * 
   * @deprecated Use {@link InplaceEditorArgumentKeys#PROPERTY_EDIT_QUALIFIED} instead.
   * In case of an attribute it will offer a clark name instead of the QName used in the CSS. 
   */
  @Deprecated
  String PROPERTY_EDIT = "edit";
  /**
   * A set of values to be presented as choices. If not present these values will be 
   * taken from the schema. The processed value for this property is a list of 
   * {@link CIValue}.
   */
  String PROPERTY_VALUES = "values";
  /**
   * Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_CHECKBOX}.
   * These are the values that are committed for a checkbox when it is un-checked.
   * If missing, an un-checked button will commit no value.
   *  
   * The processed value for this property is a list of {@link CIValue}.
   */
  String PROPERTY_UNCHECKED_VALUES = "uncheckedValues";
  /**
   * A set of labels to be associated with PROPERTY_VALUES.
   */
  String PROPERTY_LABELS = "labels";
  /**
   * A set of tooltips to be associated with PROPERTY_VALUES.
   */
  String PROPERTY_TOOLTIPS = "tooltips";
  /**
   * The number of rows that the editor should have. It's interpretation is dependent
   * to the editor being used.
   */
  String PROPERTY_ROWS = "rows";
  /**
   * The number of columns that the editor should have. It's interpretation is dependent
   * to the editor being used.
   */
  String PROPERTY_COLUMNS = "columns";
  /**
   * Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_CHECKBOX} or 
   * {@link #TYPE_POPUP_SELECTION}.
   * 
   * This is the separator that will be used to compose the values from the
   * check-boxes into one string that will be committed into the document.
   * 
   * If no separator is specified, a space will be used. 
   */
  String PROPERTY_SEPARATOR = "resultSeparator";
  /**
   * Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_POPUP_SELECTION}.
   * 
   * This is the separator that will be used to compose the values from the
   * check-boxes into one string that will be rendered in the document.
   * 
   * If no separator is specified, {@link #PROPERTY_SEPARATOR} will be used to 
   * identify tokens.
   */
  String PROPERTY_RENDERER_SEPARATOR = "rendererSeparator";
  /**
   * Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_POPUP_SELECTION}.
   * 
   * This is the order in which the values in the element or attribute will appear rendered in the editor.
   * 
   * If no sort order is specified, the values will be rendered in the order in which they appear in the document.
   */
  String PROPERTY_RENDERER_SORT = "rendererSort";
  /**
   * Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_POPUP_SELECTION} 
   * and {@link #PROPERTY_SELECTION_MODE} is set to {@link #SELECTION_MODE_MULTIPLE}.
   * 
   * This is the order in which the values specified for the element or attribute will be presented in the pop-up editor.
   * 
   * If no sort order is specified, the values will be presented in the order in which they appear in the document.
   */
  String PROPERTY_EDITOR_SORT = "editorSort";
  /**
   * Value Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_POPUP_SELECTION}.
   * 
   * Sort in ascending lexicographical order.
   */
  String SORT_ASCENDING = "ascending";
  /**
   * Value Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_POPUP_SELECTION}.
   * 
   * Sort in descending lexicographical order.
   */
  String SORT_DESCENDING = "descending";
  /**
   * Only applies when the editor is a combo box and marks the combo as being 
   * editable or not. possible values are Boolean.TRUE and Boolean.FALSE.
   */
  String PROPERTY_EDITABLE = "editable";
  /**
   * Only applies to the {@link #TYPE_BUTTON} and represents the ID of the action
   * that must be invoked when the button is pressed. It's processed value is 
   * an {@link IAuthorExtensionAction}. If an action with the given ID wasn't found 
   * the value remains the given string ID. 
   */
  String PROPERTY_ACTION_ID = "actionID";
  /**
   * Only applies to the {@link #TYPE_BUTTON} and {@link #TYPE_BUTTON_GROUP}. 
   * It specifies the context in which the action associated with the form control will be executed in.
   * 
   * The default value is {@link #ACTION_CONTEXT_ELEMENT}.
   */
  String PROPERTY_ACTION_CONTEXT = "actionContext";
  /**
   * Only applies to the {@link #TYPE_BUTTON} and ensures whether the button
   * will have a more flat appearance (transparent). The accepted values are true and false.
   * The default value is false which will determine a classic looking button.
   * When true, the SWING button will have no borders while the SWT one will be a tool item.
   * The processed values will be either Boolean.TRUE or Boolean.FALSE.
   */
  String PROPERTY_TRANSPARENT = "transparent";
  /**
   * Only applies to the {@link #TYPE_BUTTON_GROUP} and contains a list of comma separated 
   * IDs of the actions that are presented to the user in a pop-up menu. It's processed 
   * value is a list of {@link IAuthorExtensionAction}. If one of the actions IDs wasn't 
   * found the value remains the given string ID. 
   */
  String PROPERTY_ACTION_IDS = "actionIDs";
  /**
   * Only applies to the {@link #TYPE_BUTTON_GROUP} and specifies the path to an Icon
   * to be displayed on the button. The processed value is an Icon to be displayed on the button
   * or null if no icon is specified or the specified one cannot be loaded.
   */
  String PROPERTY_ICON = "icon";
  
  /**
   * Only applies to the {@link #TYPE_BUTTON} and specifies if the text should be displayed 
   * on the button. If missing, the button displays only the icon if it is available, or the 
   * text if the icon is not available. This is a boolean property. 
   */
  String PROPERTY_SHOW_TEXT= "showText";

  /**
   * Only applies to the {@link #TYPE_BUTTON} and specifies if the icon should be displayed 
   * on the button. If missing, the button displays only the icon if it is available, or the 
   * text if the icon is not available. This is a boolean property.
   */
  String PROPERTY_SHOW_ICON= "showIcon";

  /**
   * Only applies to the {@link #TYPE_BUTTON_GROUP} and specifies the label of the button
   * that triggers the pop-up menu. 
   */
  String PROPERTY_LABEL = "label";
  /**
   * Only applies to the {@link #TYPE_BUTTON_GROUP} and {@link #TYPE_TEXT} and specifies the tooltip for the button
   * that triggers the pop-up menu with the actions in the group. 
   */
  String PROPERTY_TOOLTIP = "tooltip";
  /**
   * Only applies to the {@link #TYPE_BUTTON_GROUP} and specifies what to display for an
   * action: icon, text or both. 
   */
  String PROPERTY_ACTION_DISPLAY_STYLE = "actionStyle";
  
  /**
   * Class name of the renderer. This must be a SWING implementation for both 
   * the Oxygen stand alone or Eclipse plugin version. 
   * 
   * This class will be look for in the class path of the associated document type
   * or in the {@link #PROPERTY_CLASSPATH}.
   */
  String PROPERTY_RENDERER_CLASS_NAME = "rendererClassName";
  /**
   * Class name of the editor. The SWING implementation is used for the Oxygen stand alone.
   * 
   * This class will be look for in the class path of the associated document type
   * or in the {@link #PROPERTY_CLASSPATH}.
   */
  String PROPERTY_SWING_EDITOR_CLASS_NAME = "swingEditorClassName";
  /**
   * Class name of the editor. The SWT implementation is used for the Eclipse plugin version.
   * 
   * This class will be look for in the class path of the associated document type
   * or in the {@link #PROPERTY_CLASSPATH}.
   */
  String PROPERTY_SWT_EDITOR_CLASS_NAME = "swtEditorClassName";
  /**
   * Class name of the server-side renderer used in the Author Webapp.
   * 
   * This class will be looked for in the class path of the associated document 
   * type or in the {@link #PROPERTY_CLASSPATH}.
   */
  String PROPERTY_WEBAPP_RENDERER_CLASS_NAME = "webappRendererClassName";
  /**
   * If the form control is a custom implementation this property can be used to 
   * specify the class path where the custom implementation will loaded from. 
   * A comma separated enumeration of URLs.
   */
  String PROPERTY_CLASSPATH = "classpath";
  /**
   * Used only when {@link #PROPERTY_TYPE} is set to {@link #TYPE_POPUP_SELECTION}.
   * 
   * Its possible values are {@link #SELECTION_MODE_SINGLE} and {@link #SELECTION_MODE_MULTIPLE}.
   * The default value is {@link #SELECTION_MODE_MULTIPLE}.
   */
  String PROPERTY_SELECTION_MODE = "selectionMode";
  /**
   * Property used to specify that the font is inherited from the parent element.
   * 
   * Its possible values are true or false.
   */
  String PROPERTY_FONT_INHERIT = "fontInherit";
  /**
   * Property used to specify the foreground color.
   * 
   * Its possible values are a color or 'inherit' if the color should be inherited from the element.
   */
  String PROPERTY_COLOR = "color";
  /**
   * Property used to specify the content type of the edited string.
   * 
   * The values belongs to the {@link ro.sync.exml.editor.ContentTypes}
   */
  String PROPERTY_CONTENT_TYPE = "contentType";
  /**
   * Property used to specify if an in-place editor is visible.
   * 
   * Its possible values are <code>true</code> or <code>false</code>.
   */
  String PROPERTY_VISIBLE = "visible";
  /**
   * Controls the validation of the input.
   */
  String PROPERTY_VALIDATE_INPUT = "validateInput";
  /**
   * Controls the TAB behavior in the text area form control. If this property 
   * is true, TAB is used for indentation instead of navigation.
   * 
   * By default, it is set to true.
   */
  String PROPERTY_INDENT_ON_TAB = "indentOnTab";


  //////////////////////////////////////
  
  //////////////////////////////////////
  /**
   * Possible value for {@link #PROPERTY_ACTION_CONTEXT}. The action will be executed
   * in the context of the element associated with the form control.
   * 
   * <p>
   * We want the form control below to delete the <code>li</code> element
   * even if the caret is located in a descendant of <code>li</code>. 
   * </p>
   * <pre>
   * li:before {
   *   content:oxy_button(actionID, 'delete.element', actionContext, 'element')
   * }
   * </pre>
   */
  String ACTION_CONTEXT_ELEMENT = "element";
  /**
   * Possible value for {@link #PROPERTY_ACTION_CONTEXT}. The action will be executed
   * in the current selection context. The selection/caret must be inside the element
   * associated with the form control. Otherwise the action will be executed 
   * in {@link #ACTION_CONTEXT_ELEMENT} context.
   * 
   * <p>
   * The form control is added on a 'para' element. Whenever the user makes a selection 
   * inside 'para' and clicks the button we want to wrap the existing selection in a 'b' element.
   *  
   * </p>
   * <pre>
   * para:before {
   *   content:oxy_button(actionID, 'bold.wrap', actionContext, 'caret')
   * }
   * </pre>
   */
  String ACTION_CONTEXT_CARET = "caret";
  // Editor type values.
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a combo box should be 
   * used to render and edit.
   */
  String TYPE_BUTTON = "button";
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a combo box should be 
   * used to render and edit.
   */
  String TYPE_COMBOBOX = "combo";
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a text field with 
   * content completion support should be used to render and edit.
   */
  String TYPE_TEXT = "text";
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a text area with 
   * syntax highlight support should be used to render and edit.
   */
  String TYPE_TEXT_AREA = "textArea";
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a check box panel should be 
   * used to render and edit.
   */
  String TYPE_CHECKBOX = "check";
  /**
   * The renderer presents a simple or compose value and the editor shows a 
   * pop-up like panel in which we can choose the values.
   */
  String TYPE_POPUP_SELECTION = "popupSelection";
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a URL chooser 
   * should be used to render and edit.
   * 
   * The new type of URL chooser that uses an InputUrlPanel.
   */
  String TYPE_URL_CHOOSER = "urlChooser";
  /**
   * Possible value for PROPERTY_TYPE. Indicates that a button with a pop-up menu 
   * that contains a list of actions should be used to render and edit.
   */
  String TYPE_BUTTON_GROUP = "buttonGroup";
  /////////////////////////////////////
  /**
   * Value of the PROPERTY_EDIT that indicates that the text content of the 
   * element must be edited.
   */
  String EDIT_TEXT_CONTENT = "#text";
  /**
   * <p>
   * Value of the PROPERTY_EDIT that indicates that the editor is doing a custom 
   * edit session and it's up to the editor to update the document.
   * </p> 
   * <p>In this situation it is recommended for the editor to give an {@link EditingEvent#customEdit}
   * on {@link InplaceEditingListener#commitValue(EditingEvent)}. If that's not done the following apply:</p>
   * <p>
   * In this case the notification {@link InplaceEditingListener#commitValue(EditingEvent)} will
   * do nothing since it's not clear where should the given value be committed.
   * </p>
   * <p>
   * The notification {@link InplaceEditingListener#editingStopped(EditingEvent)} will
   * also stop the editing session without committing any value. It's up to the custom
   * editor to make the necessary changes into the document (but only after the 
   * previous mentioned notification was issued).
   * </p>
   * <p>
   * <strong>Warning:</strong>
   * All changes to the document (that the custom editor must do) 
   * must be performed after the {@link InplaceEditingListener#editingStopped(EditingEvent)}
   * notification was fired. Because the editing is automatically 
   * stopped on any document modification an infinite loop will happen if the previous condition is not met.
   * </p>
   */
  String EDIT_CUSTOM = "custom";
  ////////////////////////////////////
  /**
   * Possible value for PROPERTY_EDITABLE that marks the combo as NOT being editable.
   */
  String FALSE = "false";
  /**
   * Possible value for PROPERTY_EDITABLE that marks the combo as being editable.
   */
  String TRUE = "true";
  ////////////////////////////////
  /**
   * Possible value for {@value #PROPERTY_SELECTION_MODE}. Only a single value
   * will be selected.
   */
  String SELECTION_MODE_SINGLE = "single";
  /**
   * Possible value for {@value #PROPERTY_SELECTION_MODE}. It allows multiple values 
   * to be selected.
   */
  String SELECTION_MODE_MULTIPLE = "multiple";
  /**
   * An editor that can be used to edit dates. It handles schema types xs:date and xs:datetime or 
   * any type with a specified Java format from CSS.
   * 
   */
  String TYPE_DATE_PICKER = "datePicker";
  /**
   * The old url chooser. Left here as a workaround if someone got really attached to it.
   */
  String TYPE_OLD_URL_CHOOSER = "popupUrlChooser";
  /**
   * It applies only on date picker form control. It specifies the date-time format of the edited value.
   */
  String PROPERTY_FORMAT = "format";
  
  /**
   * The variable used to add a comma in property values.
   * The real comma is used as a delimiter for multiple values thus
   * this special variable is needed.
   */
  String COMMA = "${comma}";
  
  /**
   * Value constant used for the color property.
   */
  String INHERIT = "inherit";
}