/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.schemaaware;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandler;

/**
 * Contains information about the result of the last operation handled by {@link AuthorSchemaAwareEditingHandler}.
 * The user can provide a custom implementation in order to transfer useful information about the 
 * event handling.
 * 
 * @since 11.2
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public interface SchemaAwareHandlerResult extends SchemaAwareHandlerResultInsertConstants {
  /**
   * Result generated by a keyboard delete operation (using delete or backspace keys). 
   */
  String TYPE_HANDLE_DELETE_OPERATION = "result.value.handle.delete.operation";
  
  /**
   * Result generated by a delete element tags operation.
   */
  String TYPE_HANDLE_DELETE_ELEMENT_TAGS_OPERATION = "result.value.handle.delete.element.tags.operation";
  
  /**
   * Result generated by a delete selection operation.
   */
  String TYPE_HANDLE_DELETE_SELECTION_OPERATION = "result.value.handle.delete.selection.operation";
  
  /**
   * Result generated by a join operation.
   */
  String TYPE_HANDLE_JOIN_ELEMENTS_OPERATION = "result.value.handle.join.elements.operation";
  
  /**
   * Result generated by an insert fragment operation. 
   */
  String TYPE_HANDLE_INSERT_FRAGMENT_OPERATION = "result.value.handle.insert.fragment.operation";
  
  /**
   * Result generated by a typing operation.
   */
  String TYPE_HANDLE_TYPING_OPERATION = "result.value.handle.typing.operation";
  
  /**
   * Get the result for the given id.
   * 
   * @param resultId One of the constants defined in this interface.
   * 
   * @return The value for the result. Can be <code>null</code> for an unknown result id.
   */
  Object getResult(String resultId);
  
  /**
   * The type of operation that generated the result. Depending on a result type, different information is available through
   * {@link #getResult(String)} method.
   * Possible values are:
   * <ul>
   * <li>{@link #TYPE_HANDLE_DELETE_ELEMENT_TAGS_OPERATION} for delete element tags operation, 
   * see {@link AuthorSchemaAwareEditingHandler#handleDeleteElementTags(ro.sync.ecss.extensions.api.node.AuthorNode, AuthorAccess)};</li>
   * <li>{@link #TYPE_HANDLE_DELETE_OPERATION} for a keyboard delete operation, 
   * see {@link AuthorSchemaAwareEditingHandler#handleDelete(int, int, AuthorAccess, boolean)};</li>
   * <li>{@link #TYPE_HANDLE_DELETE_SELECTION_OPERATION} for delete selection operation, 
   * see {@link AuthorSchemaAwareEditingHandler#handleDeleteSelection(int, int, int, AuthorAccess)};</li>
   * <li>{@link #TYPE_HANDLE_JOIN_ELEMENTS_OPERATION} for join elements operation, 
   * see {@link AuthorSchemaAwareEditingHandler#handleJoinElements(ro.sync.ecss.extensions.api.node.AuthorNode, java.util.List, AuthorAccess)};</li>
   * <li>{@link #TYPE_HANDLE_INSERT_FRAGMENT_OPERATION} for insert fragment operation, 
   * see {@link AuthorSchemaAwareEditingHandler#handlePasteFragment(int, ro.sync.ecss.extensions.api.node.AuthorDocumentFragment[], int, AuthorAccess)};</li>
   * <li>{@link #TYPE_HANDLE_TYPING_OPERATION} for typing operation, 
   * see {@link AuthorSchemaAwareEditingHandler#handleTyping(int, char, AuthorAccess)}.</li>
   * </ul> 
   * @return One of the constants from above, describing which schema aware operation generated the result.
   */
  String getType();
}
