/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api.editor.page.ditamap.keys;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;


/**
 * According to the:
 * 
 * http://docs.oasis-open.org/dita/v1.2/os/spec/archSpec/processing_key_references.html
 * 
 * and:
 * 
 * http://dita.xml.org/resource/dita-tc-faq-about-keys#Q2
 * 
 * if a node of a certain type makes a keyref to a topic ref on which <topicmeta> is defined, the content of the key
 * will be extracted from that particular <topicmeta>.
 * 
 * This content will appear inline in the Author page for that particular keyref.
 * It will also appear in the documentation window when the value for a keyref is computed.
 * 
 * The API will get called by Oxygen either when the content completion shows the user descriptions for each key reference
 * or when the content of a keyref is shown as static text in the editor.
 * 
 * @since 14.1
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class MetaContentProvider {

  /**
   * Get as text the keyref content.
   * 
   * For an XML like:<br/>
   * <code>
   * &lt;keyword keyref="test"/>
   * </code><br>
   * the method gets called like:
   * 
   * <code>getKeyRefContent("- topic/keyword ")</code>
   * 
   * @param nodeClass  The "@class" of the node which links.
   * @return The content or <code>null</code> if it cannot be determined.
   */
  public abstract String getContent(String nodeClass);
}
